﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EmitMapper;
using System.Runtime.InteropServices;
using System.Diagnostics;
using NLite;
using System.Threading;

namespace NLiteEmitCompare
{
    
    namespace FlatteningClass
    {
        public abstract class MapperBase : IObjectToObjectMapper
        {
            protected ModelObject _source;
            protected ModelDto _target;

            protected virtual void OnInitialize() { }
            public void Initialize()
            {
                OnInitialize();

                _source = new ModelObject
                {
                    BaseDate = new DateTime(2007, 4, 5),
                    Sub = new ModelSubObject
                    {
                        ProperName = "Some name",
                        SubSub = new ModelSubSubObject
                        {
                            IAmACoolProperty = "Cool daddy-o"
                        }
                    },
                    Sub2 = new ModelSubObject
                    {
                        ProperName = "Sub 2 name"
                    },
                    SubWithExtraName = new ModelSubObject
                    {
                        ProperName = "Some other name"
                    },
                };
            }

            public abstract void Map();
        }

        [Mapper(Category = "Flattening.Class", Name = "AutoMapper")]
        public class AutoMapperWrapper : MapperBase
        {
            protected override  void OnInitialize()
            {
                AutoMapper.Mapper.Initialize(cfg =>
                {
                    cfg.CreateMap<ModelObject, ModelDto>();
                });
                AutoMapper.Mapper.AssertConfigurationIsValid();
            }

            public override void Map()
            {
                _target =AutoMapper.Mapper.Map<ModelObject, ModelDto>(_source);
            }
        }

        //[Component]
        //public class BLToolkitMapper : MapperBase
        //{
        //    public override string Name
        //    {
        //        get { return "BLToolkitMapper"; }
        //    }

        //    protected override void OnInitialize()
        //    {
        //        base.OnInitialize();
               
        //    }

        //    public override void Map()
        //    {
        //        _target = BLToolkit.Mapping.Map.ObjectToObject<ModelDto>(_source);
        //    }
        //}

        [Mapper(Category = "Flattening.Class", Name = "NLiteMapper")]
        public class NLiteMaperWrapper : MapperBase
        {
            private NLite.Mapping.IMapper<ModelObject, ModelDto> mapper;
            protected override void OnInitialize()
            {
                base.OnInitialize();

                mapper = NLite.Mapper.CreateMapper<ModelObject, ModelDto>();
            }

            public override void Map()
            {
                _target = mapper.Map(_source);
            }
        }

        [Mapper(Category = "Flattening.Class", Name = "EmitMapper")]
        public class EmitMapperWrapper : MapperBase
        {
            ObjectsMapper<ModelObject, ModelDto> mapper;
            protected override void OnInitialize()
            {
                mapper = ObjectMapperManager.DefaultInstance.GetMapper<ModelObject, ModelDto>();
            }

            public override void Map()
            {
                _target = mapper.Map(_source);
            }
        }

        [Mapper(Category = "Flattening.Class", Name = "Manual")]
        public class ManualMapper : MapperBase
        {
            public override void Map()
            {
                var destination = new ModelDto
                {
                    BaseDate = _source.BaseDate,
                    Sub2ProperName = _source.Sub2.ProperName,
                    SubProperName = _source.Sub.ProperName,
                    SubSubSubIAmACoolProperty = _source.Sub.SubSub.IAmACoolProperty,
                    SubWithExtraNameProperName = _source.SubWithExtraName.ProperName
                };
            }
        }

     
        public class ModelObject
        {
            public DateTime BaseDate { get; set; }
            public ModelSubObject Sub { get; set; }
            public ModelSubObject Sub2 { get; set; }
            public ModelSubObject SubWithExtraName { get; set; }
        }

        public class ModelSubObject
        {
            public string ProperName { get; set; }
            public ModelSubSubObject SubSub { get; set; }
        }

        public class ModelSubSubObject
        {
            public string IAmACoolProperty { get; set; }
        }

        public class ModelDto
        {
            public DateTime BaseDate { get; set; }
            public string SubProperName { get; set; }
            public string Sub2ProperName { get; set; }
            public string SubWithExtraNameProperName { get; set; }
            public string SubSubSubIAmACoolProperty { get; set; }
        }
    }

    public sealed class CodeTimer
    {
        public static void Initialize()
        {
            Process.GetCurrentProcess().PriorityClass = ProcessPriorityClass.High;
            Thread.CurrentThread.Priority = ThreadPriority.Highest;
            Time("", 1, () => { });
        }

        public static void Time(string name, Action action)
        {
            Time(name, 1, action);
        }

        public static void Time(string name, int iteration, Action action)
        {
            if (String.IsNullOrEmpty(name)) return;

            // 1.
            ConsoleColor currentForeColor = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.WriteLine(name);

            // 2.
            GC.Collect(GC.MaxGeneration, GCCollectionMode.Forced);
            int[] gcCounts = new int[GC.MaxGeneration + 1];
            for (int i = 0; i <= GC.MaxGeneration; i++)
            {
                gcCounts[i] = GC.CollectionCount(i);
            }

            // 3.
            Stopwatch watch = new Stopwatch();
            watch.Start();
            long cycleCount = GetCycleCount();
            for (int i = 0; i < iteration; i++) action();
            long cpuCycles = GetCycleCount() - cycleCount;
            watch.Stop();

            // 4.
            Console.ForegroundColor = currentForeColor;
            Console.WriteLine("\tTime Elapsed:\t" + watch.ElapsedMilliseconds.ToString("N0") + "ms");
            Console.WriteLine("\tCPU Cycles:\t" + cpuCycles.ToString("N0"));

            // 5.
            for (int i = 0; i <= GC.MaxGeneration; i++)
            {
                int count = GC.CollectionCount(i) - gcCounts[i];
                Console.WriteLine("\tGen " + i + ": \t\t" + count);
            }

            Console.WriteLine();

        }

        private static long GetCycleCount()
        {
            //ulong cycleCount = 0;
            //QueryThreadCycleTime(GetCurrentThread(), ref cycleCount);
            //return cycleCount;
            return GetCurrentThreadTimes();
        }

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool GetThreadTimes(IntPtr hThread, out long lpCreationTime,
           out long lpExitTime, out long lpKernelTime, out long lpUserTime);

        private static long GetCurrentThreadTimes()
        {
            long l;
            long kernelTime, userTimer;
            GetThreadTimes(GetCurrentThread(), out l, out l, out kernelTime,
               out userTimer);
            return kernelTime + userTimer;
        }


        //[DllImport("kernel32.dll")]
        //[return: MarshalAs(UnmanagedType.Bool)]
        //static extern bool QueryThreadCycleTime(IntPtr threadHandle, ref ulong cycleTime);

        [DllImport("kernel32.dll")]
        static extern IntPtr GetCurrentThread();

    }

}
