﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using NLite;

namespace NLiteEmitCompare
{
    [Contract]
    public interface IObjectToObjectMapper
    {
        //初始化映射器
        void Initialize();
        //执行映射
        void Map();
    }

    //测试映射器元数据
    public interface IMapperMetadata
    {
        //目录
        string Category { get; }
        //名称
        string Name { get; }
        string Descrption { get; }
    }

    //映射器元数据注解
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false)]
    [MetadataAttributeAttribute]
    public class MapperAttribute : ComponentAttribute
    {
        public string Category { get; set; }
        public string Name { get; set; }
        public string Descrption { get; set; }
    }


    class Program
    {
        [InjectMany]
        private Lazy<IObjectToObjectMapper,IMapperMetadata>[] Mappers;

        //初始化映射器，并做一次映射操作
        void Init()
        {
            foreach (var item in Mappers)
            {
                item.Value.Initialize();
                item.Value.Map();
            }

        }

        //进行测试
        void Run()
        {
            foreach (var item in Mappers)
                CodeTimer.Time(item.Metadata.Category +"->" + item.Metadata.Name , 100000,() => item.Value.Map());
        }

        static void Main(string[] args)
        {
            ServiceRegistry.RegisteryFromAssemblyOf<Program>();

            var host = new Program();
            ServiceRegistry.Compose(host);

            host.Init();
            host.Run();

            Console.Read();
        }
    }
}
